function [num_result,text_result,row,columns,cr_input,relevant_data_f,zern] = APHEFIS_bridge(num_peaks,zoomfa,compare,z_coefs,fast,pixel_focalfield,apdiameter_meter,fieldsize_m,lambda,iangle,base_field,iterations,fake_metric,zernoptions,param,prev_inputs)
% APHEFIS_bridge, based in AO_APHEFIS, based on "AO_focal_one.M" by D. Gigenbach, based on "fieldstatistics.m" by M. Gdel
% By Manolo Munoz, started on 2011/09/28
% Starting from a given apperture field, statistics of it are made and the 
% process for obtaining the correction by the method of estimating it's
% phase by the intensity speckles. 
% Now in process of evaluation of the different options for good
% behaviour and performance
%
%

%------------------------ set input parameter -----------------------------

Zernike_size=64;
focus = 1; % focal length in [m]
zeropad_factor = 4; % factor to increase the aperture matrix by zeropadding [1,2,4]

input_args={'index','num_levels','nearestpeaks','num_peaks','zoomfa','graphics','compare','time','z_coefs','min_factor',...
    'pixel_focalfield','apdiameter_meter','lambda','fieldsize_m','iterations','nfields','iangle',...
    'base_field','metric','fake_metric','substract_maximum_phase','zernoptions','threshold'};

zern=[];
th_HE=0.3;
th_Strehl2=0.55;
fake_metric=fix(fake_metric);

if isstruct(prev_inputs)
    cr_input=prev_inputs;
    siz=size(cr_input.data,1);
else
    siz=0;
end

if base_field==0
    base_field=randi(999)+1;
end

if fast
    iterations=1;
    fake_metric=1;
end

if (fake_metric~=0)&&(fast==0)
    fast=1+fake_metric;
end

%It's made structure with the names of the input characteristics and the
%value of them. It will be usefull for maquing stadistics. For not working
%two times, I pass the function the previous values, so we can simply update 
%the values of it 

columns={'TotalP','ZonesP','AsignedP','CorrectP','BestAsignP','InStrehl','OutStrehl','InHE','OutHE','MaximumPhase','PerfectHE','PerfectStrehl','Scintilation','Branchpoints_csm','r0'};
num_result=zeros(iterations,size(columns,2));
rows=cell(iterations,1);

field_dir = strcat('..\Downlink_1550nm\',num2str(iangle),'_deg\256er_1550\');

field_file = strcat(field_dir,'Num_',num2str(base_field),'_fieldExt_in_m_1.024_PixelNum_256.mat');


text_result=[];
pause on;


%     images_row=3;
% %     images={'','abs(focusfield)','abs(E_corrected_focus)','angle(field_apmask)','abs(fairy)'};
%     images={'','phase_corrected','phase_correction','correct_phase','angle(field_apmask)'};
    images_row=4;
    images={'','angle(field_apmask)','phase_corrected','phase_correction','watersheed_intensity','abs(E_corrected_focus)','af'};
%     images={'','abs(E_corrected_focus)','imag(field_apmask)','real(focusfield)','imag(focusfield)','abs(focusfield)','angle(focusfield)'};
%     images={'','angle(field_apmask)','unwr','phase_correction','correct_phase','abs(focusfield)','abs(E_corrected_focus)'};
   
%%% Figure-Position for field-display with 5 subplots: upos = [posXF, posYstartF, widthF, heightF];
XcenterF=(1050);widthF=320; heightF=170;
widthF=520; heightF=370;

cm_rg=zeros(20,3); 
cm_rg(1:10,1)=.9:-.075:0.225;%  red for -pi to 0
cm_rg(11:20,2)=0.225:.075:.9;%  and green for 0 to +pi



cr_input.columns=input_args;

for i=1:size(input_args,2)
    if siz==0&&strcmp(input_args{i},'threshold')==0
        eval(['cr_input.i',input_args{i},'=i;']);
    end
    for k=1:iterations
                if strcmp(input_args{i},'iterations')
                    cr_input.data(siz+k,cr_input.iiterations)=k;
                elseif strcmp(input_args{i},'base_field')
                    cr_input.data(siz+k,cr_input.ibase_field)=base_field;
                elseif strcmp(input_args{i},'threshold')
                    cr_input.threshold{siz+k}='';
                elseif strcmp(input_args{i},'z_coefs')
                    if isstruct(z_coefs)
                        zern=z_coefs;
                        size(zern.ain{1})
                        z_coefs=size(zern.ain{1},1);
                    end
                    cr_input.z_coefs{siz+k}=z_coefs;
                elseif strcmp(input_args{i},'index')==0&&exist(input_args{i},'var')
                    eval(['cr_input.data(siz +k,cr_input.i',input_args{i},')=',input_args{i},';']);
                elseif strcmp(input_args{i},'index')==0
                    eval(['cr_input.data(siz +k,cr_input.i',input_args{i},')=0;']);
                end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Opening and Resizing %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



field = load(field_file); 
field = field.rxfield;


    field=cut_image(field,apdiameter_meter/fieldsize_m);
    pixel_focalfield=Zernike_size;
    field=imresize(field,[pixel_focalfield,pixel_focalfield]);
    fieldsize_m=apdiameter_meter;
    
    
%------------------------ start calculation -------------------------------

field_size = size(field,1); 
ff_cut = pixel_focalfield; 

% dxa is dynamically calculated from the matrix size of the input field
dxa = fieldsize_m/field_size;   %  in [meter/pixel]
apdiameter = apdiameter_meter  / dxa; 

mask = apmask_v2(apdiameter,field_size);
% field=mask.*exp(1i*angle(field));

field_apmask=field;
ap_sup=sum(sum(mask*dxa^2));
mask2=mask/(sum(sum(mask.^2))*ap_sup)^0.5;
field_apmask = field_apmask.*mask;            
field_apmask=field_apmask/(sum(sum(abs(field_apmask).^2))*ap_sup)^0.5;
[focusfield, dxf] = focustransformation(field_apmask, zeropad_factor, lambda, focus, dxa, ff_cut);
focus_sup=ap_sup*(dxf/dxa)^2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Tip-tilt Correction %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if fast~=1

[focusfield_full, dxf_full] = focustransformation(field_apmask, 1, lambda, focus, dxa, ff_cut);

% focus_amp = abs(focusfield);
% focus_int = focus_amp.^2;
% [COGy,COGx] = COGpixels(abs(focusfield).^2);   
% 
% center(2) = COGx-pixel_focalfield/2-1;       
% center(1) = COGy-pixel_focalfield/2-1; 
% 
% 
%     
% 
% center=(center-(size(focus_int))/2-1)
% 
%     
% % if max(abs(center))>1
%     center=center./size(focus_int);
% 
%     tt=(exp(2*pi*1i*center(1)*[1:size(focus_int)])'*exp(-2*pi*1i*center(2)*[1:size(focus_int)]));
% 
%     field_apmask=field_apmask.*tt;
%     ang_apfield=angle(field_apmask);
%     unwr=hm_unwrap(ang_apfield,mask,pi);
% %     if (sum(ang_apfield(:)>pi/2)+sum(ang_apfield(:)<-pi/2))>(field_size^2)/4
% %         ang_apfield=ang_apfield+pi;
% %         ang_apfield=ang_apfield+(ang_apfield>pi)*2*pi;
%         field_apmask=abs(field_apmask).*exp(1i*ang_apfield);
% %     end
%     [focusfield, dxf] = focustransformation(field_apmask, zeropad_factor, lambda, focus, dxa, ff_cut);
% % end
% 
% focus_amp = abs(focusfield);
% focus_int = focus_amp.^2;
% 
% [focusfield_full, dxf_full] = focustransformation(field_apmask, 1, lambda, focus, dxa, ff_cut);
% center(1)=sum([1:size(focus_int,1)]*abs(focusfield_full).^2);
% center(2)=sum(abs(focusfield_full).^2*[1:size(focus_int,2)]');
% center=center/sum(abs(focusfield_full(:)).^2);
% 
% 
%     
% 
% center=(center-(size(focus_int))/2-1)
% 
% 
% field_data.aperture=field_apmask;
% field_data.focus=focusfield;
% save('.\data_in.mat','field_apmask', 'focusfield');
% 
% field=field_apmask;

end

abs_apfield=abs(field_apmask);
ap_int=abs_apfield.^2;
Scintilation=var(ap_int(mask~=0))/(mean(ap_int(mask~=0))^2);
Scintilation(isnan(Scintilation))=0;


if fast~=1
    [perfectfocusfield, dxf] = focustransformation(abs_apfield, zeropad_factor, lambda, focus, dxa, ff_cut);
    focus_sup_full=ap_sup*(dxf_full/dxa)^2;
    perfectfocus_amp=abs(perfectfocusfield);
    perfectfocus_int=perfectfocus_amp.^2;
end
[fairy, ~] = focustransformation(mask2, zeropad_factor, lambda, focus, dxa, ff_cut);

afairy=angle(fairy);
intairy=abs(fairy).^2;
% 
% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Temp %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% 
% zern.ain{1}=zeros(50,1);
% zern.ain{1}(1:10,1)=0.5*ones(10,1);
% % zern.ain{1}(4,1)=1;
% zern.ain{1}(7,1)=1;
% % zern.ain{1}(16,1)=1;
% % zern.ain{1}(11,1)=1;
% zern.rmsi=0;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Zernike related operations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if fast~=1
zcorr=rem(zernoptions,10);
parity=round(zernoptions/100);
zernoptions=(zernoptions-zcorr)/10-10*parity;



if (z_coefs>0)
    if (~isstruct(zern))
        [zern.ain{1},phiRec1,zern.rmsi(1)]=MatchZernike(GoldsteinUnwrap2D(field),z_coefs);
    else
        a=zern.ain{1}.';
        phiRec1=zeros(pixel_focalfield);
        for i=1:z_coefs
            phiRec1=phiRec1+a(i)*zernimat(pixel_focalfield,i);
        end;  
    end
    if parity
        a=zern.ain{1}.';
        zern.ain{1}=zeros(size(a));
        phiRec=zeros(pixel_focalfield);
        i=parity;
        cicle=parity+2;
        maxi=2*parity;
        while i<z_coefs
            if i<maxi
                phiRec=phiRec+a(i)*zernimat(pixel_focalfield,i);
                zern.ain{1}(i,1)=a(i);
                i=i+1;
            else
                i=i+cicle-1;
                maxi=i+cicle;
                cicle=cicle+2;
            end
    
        end;
    
        phiRec1=phiRec;
    end
end

% figure;imagesc(phiRec1);

if zernoptions==1
    field=abs(field_apmask).*exp(1i*phiRec1);
    field_apmask=field;
    [focusfield, dxf] = focustransformation(field_apmask, zeropad_factor, lambda, focus, dxa, ff_cut);
%      figure;imagesc(abs(focusfield))
elseif zernoptions>1
    field=abs(field_apmask).*exp(1i*(angle(field)-phiRec1));
    field_apmask=field;
    [focusfield, dxf] = focustransformation(field_apmask, zeropad_factor, lambda, focus, dxa, ff_cut);
end

end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Loop throught iterations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




for it=1:iterations
    
    
    
%%%%%%%%%%%%%%%Focus Transformation%%%%%%%%%%%%%%%%%%%


focus_amp = abs(focusfield);
focus_int = focus_amp.^2;    

% if it>1
% % 
% %     center(1)=sum([1:size(focus_int,1)]*focus_int);
% %     center(2)=sum(focus_int*[1:size(focus_int,2)]');
% % 
% %     center=center/sum(sum(focus_int));
%     [~,center(1)]=max(max(focus_int,[],2));
%     [~,center(2)]=max(max(focus_int,[],1));
%     center=center-(size(focus_int))/2-1
%     if max(abs(center))>=1
%         center=center./size(focus_int)/4;
% 
%         tt=(exp(2*pi*1i*center(1)*[1:size(focus_int)])'*exp(-2*pi*1i*center(2)*[1:size(focus_int)]));
% 
%         field_apmask=field_apmask.*tt;
%         [focusfield, dxf] = focustransformation(field_apmask, zeropad_factor, lambda, focus, dxa, ff_cut);
%     end
%     
% end

%     Potency_no_zeropad_in=sum(sum((abs(focusfield_full).^2)))*focus_sup_full;

    focus_phase = angle(focusfield);
    maximum_phase=focus_phase(focus_amp==max(max(focus_amp)));

    maxIfocus_orig=max(max(focus_int));
    maxIairy=max(max(abs(fairy).^2));
    HE_apfield = HE_Aperture(field_apmask,apdiameter); % With apdiameter in [m]        
    strehl_aperture=maxIfocus_orig/maxIairy;
    if fast~=1
        perfectStrehl=max(max(perfectfocus_int))/maxIairy;
        perfectHE= HE_Aperture(abs_apfield,apdiameter);
    end
    watersheed_intensity=focus_int;
    amp_difference=(focus_amp-abs(fairy)*maxIfocus_orig/maxIairy);
    amp_difference=abs(amp_difference);
 
%     if (strehl_aperture>th_Strehl2)&&fast~=1
%         fast=20; 
%     end

    % fwhm_m = calculate_fwhm_m(focus_int,dxf); 
    % r0_fwhm = lambda/fwhm_m * 0.98 * focus;        
        
    focus_phase=focus_phase-maximum_phase(1)+pi/2;
    focus_phase(focus_phase<-pi)=focus_phase(focus_phase<-pi)+2*pi;
    focus_phase(focus_phase>pi)=focus_phase(focus_phase>pi)-2*pi;
    afairy=afairy+pi/2;
    afairy(afairy<-pi)=afairy(afairy<-pi)+2*pi;
    afairy(afairy>pi)=afairy(afairy>pi)-2*pi;

    full_correct_phase = (round(((focus_phase/pi*2+1)/2))*2-1)*pi/2;
    full_correct_phase(full_correct_phase<=-pi)=full_correct_phase(full_correct_phase<=-pi)+2*pi;
    full_correct_phase(full_correct_phase>pi)=full_correct_phase(full_correct_phase>pi)-2*pi;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Aply the correction %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if fake_metric==4
        num_peaks=250;
    end

    [phase_correction,relevant_data,shed_int,peak_values,E_backtrafo,focus_phase_corr_E,text1]=APHEFIS_CORRECTION(watersheed_intensity,param,num_peaks,fast,dxa, lambda, focus, zeropad_factor,apdiameter_meter,focus_sup,afairy);
    text_result=[text_result text1 13];
    phase_correction =phase_correction.*mask; 
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Aply the correction %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    base_phase=zeros(size(focus_phase_corr_E));

%     if HE_apfield>th_HE
%         base_phase=afairy+pi/2;
%         base_phase(base_phase>pi)=base_phase(base_phase>pi)-2*pi;
%         focus_phase_corr_E(focus_phase_corr_E==0)=angle(fairy(focus_phase_corr_E==0))+pi/2;   
%         focus_phase_corr_E(focus_phase_corr_E>pi)=focus_phase_corr_E(focus_phase_corr_E>pi)-2*pi;
%     end

    best_possible_phase_st=base_phase;
    airy_phase_st=base_phase;
    mean_phase_zones=zeros(size(full_correct_phase));
    relevant_data.best_phase_vec=zeros(size(peak_values,1),1);
    mean_phase=zeros(size(peak_values,1),1);
    mean_airy_phase=zeros(size(peak_values,1),1);
    for k=1:size(peak_values,1) 
        %   we calculate the sign that should be, to see how good our system could be
        mean_phase(k,1)=angle(sum(sum(exp(1i*focus_phase(shed_int==shed_int( peak_values(k,1) , peak_values(k,2) ))).*focus_int(shed_int==shed_int( peak_values(k,1) , peak_values(k,2) )))));
        relevant_data.best_phase_vec(k,1) = (round(((mean_phase(k,1)/pi*2+1)/2))*2-1)*pi/2;
        relevant_data.best_phase_vec(k,1)=relevant_data.best_phase_vec(k,1)-2*pi*(relevant_data.best_phase_vec(k,1)>pi)+2*pi*(relevant_data.best_phase_vec(k,1)<=-pi);        
        mean_phase_zones(shed_int==shed_int( peak_values(k,1) , peak_values(k,2) ))=mean_phase(k,1);
        airy_phase_st(shed_int==shed_int( peak_values(k,1) , peak_values(k,2) ))=relevant_data.airy_phase_vec(k,1);
        
        best_possible_phase_st(shed_int==shed_int( peak_values(k,1) , peak_values(k,2) ))=relevant_data.best_phase_vec(k,1);
        relevant_data.true_phase(k)=sign(focus_phase(peak_values(k,1) , peak_values(k,2)));
        
    end
    text_result=[text_result,char(10),['Ideal phase       = ' num2str(sign(relevant_data.true_phase)) char(10) 'Asigned phase = ' num2str(relevant_data.phase_vec') ],char(10)];


    relevant_data.apdiameter_meter=apdiameter_meter;
    relevant_data.iangle=iangle;
    relevant_data.fieldn=base_field;
    
    correct_phase=full_correct_phase;
    correct_phase(shed_int==0)=0;
    restricted_phase=focus_phase;
    restricted_phase(best_possible_phase_st==0)=0;
    
    if fake_metric~=0
        if fake_metric==1||fake_metric==4
            focus_phase_corr_E=best_possible_phase_st;% best phase assignment with one sign for zone
        elseif fake_metric ==2
            focus_phase_corr_E=correct_phase; % best phase assignment with correct phase assigned to the zones
        elseif fake_metric ==3
            focus_phase_corr_E=full_correct_phase;
        end
        focus_amp(focus_phase_corr_E==0)=0;
%        focus_amp(shed_int==0)=0;
        
        E_backtrafo = backtransformation(focus_amp.* exp(1i*focus_phase_corr_E), dxa, lambda, focus, zeropad_factor); 
        E_backtrafo =E_backtrafo.*mask; 
        phase_correction=angle(E_backtrafo);
%         phase_correction =phase_correction.*mask; 
    end
    focusfield(shed_int==0)=0;
    if fast==20
        phase_correction=zeros(size(focus_int));
        E_backtrafo=zeros(size(focus_int));
    end
    if fast~=1
        %columns={'TotalP','ZonesP','AsignedP','CorrectP','BestAsignP','InStrehl','OutStrehl','InHE','OutHE','MaximumPhase','PerfectHE','PerfectStrehl','Branchpoints_amp','Branchpoints_csm','r0'};
        if (zcorr==1)&&(z_coefs>0)
            phase_correction=phiRec1;
        end
        phase_correction=phase_correction-(phase_correction(round(size(focus_int,1)/4),round(size(focus_int,1)/4))+phase_correction(round(size(focus_int,1)/4)*3,3*round(size(focus_int,1)/4)))/2;
        phase_correction=(phase_correction-2*pi*(phase_correction>(pi))+2*pi*(phase_correction<=(-pi))).*mask;
        save('fc.mat','phase_correction'); 
        phase_corrected = (zeroPadMK(angle(field_apmask),size(E_backtrafo,1),0,'center') - phase_correction);
        phase_corrected=phase_corrected-2*pi*(phase_corrected>(pi))+2*pi*(phase_corrected<=(-pi));
        E_corrected_field = abs_apfield .* exp(phase_corrected * 1i);
%         figure;imagesc(phase_correction)
        
%         if ~exist(['./Phases/Inputs128/' num2str(iangle) 'degrees/Aperture' num2str(apdiameter_meter) 'm/Inputfield_' num2str(base_field) '.mat'],'file')
%             input=imresize(angle(field_apmask),[128 128]);
%             output=imresize(phase_correction,[128 128]);
%             ['./Phases/Inputs128/' num2str(iangle) 'degrees/Aperture' num2str(apdiameter_meter) 'm/Inputfield_' num2str(base_field) '.mat']
%             save(['./Phases/Inputs128/' num2str(iangle) 'degrees/Aperture' num2str(apdiameter_meter) 'm/Inputfield_' num2str(base_field) '.mat'],'input');
%             save(['./Phases/Outputs128/' num2str(iangle) 'degrees/Aperture' num2str(apdiameter_meter) 'm/Outputfield_' num2str(base_field) '.mat'],'output');
%         end
        

        E_corrected_focus = focustransformation(E_corrected_field.*mask, zeropad_factor, lambda, focus, dxa, ff_cut);
        focus_corrected_phase =angle(E_corrected_focus);
        maxIfocus_corr=max(max(abs(E_corrected_focus).^2));
        strehl_corrected=maxIfocus_corr/maxIairy;
%         tic
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Iterative method %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         if fake_metric==0
%             focus_phase_corr_E2=focus_phase_corr_E;
%             for loop=2:num_peaks
%                 focus_phase_corr_E2(shed_int==shed_int( peak_values(loop,1) , peak_values(loop,2) ))=-focus_phase_corr_E(peak_values(loop,1) , peak_values(loop,2) );
%                 E_backtrafo2 = backtransformation(focus_amp.* exp(1i*focus_phase_corr_E2), dxa, lambda, focus, zeropad_factor); 
%                 E_backtrafo2 =E_backtrafo2.*mask; 
%                 phase_correction2=angle(E_backtrafo2);
%                 phase_corrected2 = (zeroPadMK(angle(field_apmask),size(E_backtrafo,1),0,'center') - phase_correction2);
%                 phase_corrected2=phase_corrected2-2*pi*(phase_corrected2>(pi))+2*pi*(phase_corrected2<=(-pi));
%                 E_corrected_field2 = abs_apfield .* exp(phase_corrected2 * 1i);
%                 E_corrected_focus2 = focustransformation(E_corrected_field2.*mask, zeropad_factor, lambda, focus, dxa, ff_cut); 
%                 maxIfocus_corr2=max(max(abs(E_corrected_focus2).^2));
%                 strehl_corrected2=maxIfocus_corr2/maxIairy;
%                 if strehl_corrected2>strehl_corrected
%                     focus_phase_corr_E=focus_phase_corr_E2;
%                     E_backtrafo=E_backtrafo2;
%                     phase_correction=phase_correction2;
%                     phase_corrected=phase_corrected2;
%                     E_corrected_field=E_corrected_field2;
%                     strehl_corrected=strehl_corrected2;
%                     relevant_data.phase_vec(loop,1)=-relevant_data.phase_vec(loop,1);
%                     text_result=[text_result,char(10),['Ideal phase       = ' num2str(sign(relevant_data.true_phase)) char(10) 'Asigned phase = ' num2str(relevant_data.phase_vec') ],char(10)];
%                 else
%                     focus_phase_corr_E2=focus_phase_corr_E;
%                 end
%             end
%         end
%         toc
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        E_corrected_focus_full = focustransformation(E_corrected_field.*mask, 1, lambda, focus, dxa, ff_cut);
        HE_E = HE_Aperture(E_corrected_field,apdiameter);
        

        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Calculate the rings parameters (not currently used) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if compare%&&(it==iterations)
            
            cm_rg=zeros(20,3); cm_rg(1:10,1)=.9:-.075:0.225; cm_rg(11:20,2)=0.225:.075:.9;
            
            error_angle=focus_phase;%afairy-focus_phase;
            error_angle=error_angle-error_angle(size(error_angle,1)/2,size(error_angle,2)/2);
            error_angle(error_angle>pi)=error_angle(error_angle>pi)-2*pi;
            error_angle(error_angle<=-pi)=error_angle(error_angle<=-pi)+2*pi;  
            error_amp=abs(amp_difference);
%             amp_difference(1,1)=-min(min(amp_difference));
%             amp_difference(2,1)=-max(max(amp_difference));
            min_p=0;
            current_p=0.00001;
            maxpos=0; j=1;
            m=field_size/2;
            maxrings=4;
            posrings(j)=1;
            while (j<=maxrings||current_p<min_p)&&(m~=maxpos)
                maxpos=maxpos+1;
                if abs(afairy(m+maxpos+1,m)-afairy(m+maxpos,m))>0.2
                    j=j+1;
                    posrings(j)=maxpos;
                end                  
            end
            
            radp=polartrans_mod(error_angle,posrings(j),50);
            rada=polartrans_mod(error_amp,posrings(j),50);
            phase_rep=zeros(size(posrings,2)-1,size(radp,2));
            amp_rep=zeros(size(posrings,2)-1,size(rada,2));
            for j=2:size(posrings,2)
                aux1=mean(radp(posrings(j-1):posrings(j),:));
                aux2=mean(radp(posrings(j-1):posrings(j),:)+2*pi)-2*pi;
                aux1(sum(abs(aux1-radp(round((posrings(j-1)+posrings(j))/2),:)))>sum(abs(aux2-radp(round((posrings(j-1)+posrings(j))/2),:))))=aux2(sum(abs(aux1-radp(round((posrings(j-1)+posrings(j))/2),:)))>sum(abs(aux2-radp(round((posrings(j-1)+posrings(j))/2),:))));
%                 aux1(aux1<0)=aux1(aux1<0)+2*pi;
                phase_rep(j-1,:)=aux1;
                amp_rep(j-1,:)=mean(rada(posrings(j-1):posrings(j),:));
            end
            phase_deriv_rep=[phase_rep(:,size(phase_rep,2)),phase_rep(:,1:(size(phase_rep,2)-1))]-phase_rep;
            phase_deriv_rep(phase_deriv_rep>pi)=phase_deriv_rep(phase_deriv_rep>pi)-2*pi;
            phase_deriv_rep(phase_deriv_rep<=-pi)=phase_deriv_rep(phase_deriv_rep<=-pi)+2*pi;
            amp_deriv_rep=[amp_rep(:,size(phase_rep,2)),amp_rep(:,1:(size(phase_rep,2)-1))]-amp_rep;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Graphic representation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
            phase_rep_n=phase_rep./(max(abs(phase_rep),[],2)*ones(1,size(phase_rep,2)));
            amp_rep_n=amp_rep./(max(abs(amp_rep),[],2)*ones(1,size(phase_rep,2)));
            phase_deriv_rep_n=phase_deriv_rep./(max(abs(phase_deriv_rep),[],2)*ones(1,size(phase_rep,2)));
            amp_deriv_rep_n=amp_deriv_rep./(max(abs(amp_deriv_rep),[],2)*ones(1,size(phase_rep,2)));
            pos=[750,0,1650/2,1200/size(amp_rep,1)];
%             for j=1:size(amp_rep,1)
%                 figure('Name',['Ring number ' num2str(j) ],'Position',[pos(1),pos(2)+(j-1)*pos(4),pos(3),pos(4)-5]);
%                 hold on;
%                 x1=[2*pi:-pi/25:pi/25];
%                 xlabel('Angle');
%                 ylabel('Value');
%                 plot(x1,amp_rep_n(j,:),'-dm','MarkerSize',2,'MarkerFaceColor','m');
% %                 pause
%                 plot(x1,phase_rep_n(j,:),'-dc','MarkerSize',2,'MarkerFaceColor','c');
% %                 pause
% %                 plot(x1,phase_deriv_rep_n(j,:),'-db','MarkerSize',2,'MarkerFaceColor','b');
% %                 pause
% %                 plot(x1,amp_deriv_rep_n(j,:),'-dr','MarkerSize',2,'MarkerFaceColor','r');
% %                 pause
%                 legend('Amplitude Error','Phase Error','Derivative of Phase error','Derivative of Amplitude error');
% %                 pause
%                 hold off
%             end
            
            
            estimation=(angle(E_backtrafo)-pi/2*mask);
            estimation(mask==0)=0;
            estimation(estimation>pi)=estimation(estimation>pi)-2*pi;
            estimation(estimation<-pi)=estimation(estimation<-pi)+2*pi;
            estimation=(angle(E_backtrafo)-pi/2*mask);
            
            af=(angle(focusfield)-maximum_phase(1)+pi/2).*(focusfield~=0);
            af(mask==0)=0;
            af(af>pi)=af(af>pi)-2*pi;
            af(af<-pi)=af(af<-pi)+2*pi; 
            
            num_images=size(images,2);
            n_im_rows=floor((num_images)/images_row)+1;
            f=figure;
            upos = [XcenterF-min(num_images+1,images_row)*widthF/2,30+(it-1)*n_im_rows*heightF,min(num_images+1,images_row)*widthF,n_im_rows*heightF]; set(f,'Position',upos);set(f,'Name',['Field ',num2str(base_field),', Iteration ',num2str(it)]); set(gcf,'Color',[1 1 1]); %set(gcf,'Colormap', [summer]);
            
            
            
            full_correct_phase(1,1)=pi;
            full_correct_phase(1,2)=-pi;
            correct_phase(1,1)=pi;
            correct_phase(1,2)=-pi;
            best_possible_phase_st(1,1)=pi;
            best_possible_phase_st(1,2)=-pi;
            airy_phase_st(1,1)=pi;
            airy_phase_st(1,2)=-pi;
            focus_phase_corr_E(1,1)=pi;
            focus_phase_corr_E(1,2)=-pi;
            restricted_phase(1,2)=-pi;
            restricted_phase(1,1)=pi;
            mean_phase_zones(1,1)=pi;
            mean_phase_zones(1,2)=-pi;
            
            
            subplot(n_im_rows,images_row,1); imagesc(focus_phase_corr_E);colorbar; caxis manual; caxis([-pi pi]);axis square; title('Estimated Focal Phase'); zoom(zoomfa); hold on
            for j = 1 : size(peak_values,1)  
                text(peak_values(j,2) , peak_values(j,1), num2str(j),'Color',[1 1 1]); 
            end
            contour(focus_int,3,'k');
%             hold off
            
            for i=2:num_images
                ax=subplot(n_im_rows,images_row,i); eval(['imagesc(',images{i},');']);colorbar; axis square;title([transformtitle(images{i})]);zoom(zoomfa);
                if strcmp(images{i},'correct_phase')||strcmp(images{i},'full_correct_phase')||strcmp(images{i},'af')
                    colormap(ax,cm_rg);
                    hold on; caxis manual; caxis([-pi pi]); contour(focus_amp,5,'k');   hold off;
                elseif strcmp(images{i},'abs(focusfield)')||strcmp(images{i},'focus_int')||strcmp(images{i},'log10(abs(focusfield)/mean(abs(focusfield(:)))')
                    hold on; contour(fairy+0.1,[0.1 0.1],'w');   hold off;
                elseif strcmp(images{i},'focus_corrected_phase')
                    colormap(ax,cm_rg);
                    hold on; contour(abs,7,'k');caxis manual; caxis([-pi pi]);   hold off;
                end
                
            end
            
            
            
            subplot(n_im_rows,images_row,i+1); deltaYtext=.1;  cla; axis off;
            text1=['\bfInput: \rm'];
            if zernoptions==1
                text1=[text1 'Zernike with ' num2str(z_coefs) ' coeficients ']
                if parity==1
                    text1=[text1 '(only even)'];
                elseif parity==2
                    text1=[text1 '(only odd)'];
                end
            elseif zernoptions>1
                text1=[text1 'Zernike error with ' num2str(z_coefs) ' coeficients '];
                if parity==1
                    text1=[text1 '(only even)'];
                elseif parity==2
                    text1=[text1 '(only odd)'];
                end
            else
               text1=[text1 'Real input'];
            end
            text(-.05,1,text1); 
            
            text1=['\bfTreatment: \rm'];
            if zcorr==1
                text1=[text1 'Zernike with ' num2str(z_coefs) ' coeficients '];
                if parity==1
                    text1=[text1 '(only even)'];
                elseif parity==2
                    text1=[text1 '(only odd)'];
                end
            elseif fake_metric>0
               text1=[text1 'Fake Metric ' num2str(fake_metric)];
            else
                text1=[text1 'Neural Network'];
            end
            text(-.05,1-2*deltaYtext,text1); 
            
            
            text(-.05,1-4*deltaYtext,'\bfStadistics:\rm'); 
            text(0,1-5*deltaYtext,['HetEff aperture original = ',num2str(HE_apfield)]); 
            text(0,1-6*deltaYtext,['HetEff treated signal = ',num2str(HE_E)]);                  
            text(0,1-7*deltaYtext,['Strehl factor original = ',num2str(strehl_aperture)]); 
            text(0,1-8*deltaYtext,['Strehl factor treated = ',num2str(strehl_corrected)]);                                    
            
            colormap('default');
            
        end
        [~,branchpoints_csm,~,r0_method2]=stadistics_ifield(field, dxa,iangle,'', 0,0 ,0);
        nr_branchpoints_csm_ap = sum(sum(branchpoints_csm.*mask ));

        num_result(it,:)=[0,0,0,0,0,strehl_aperture,strehl_corrected,HE_apfield,HE_E,maximum_phase(1),perfectHE,perfectStrehl,Scintilation,nr_branchpoints_csm_ap,r0_method2];
        text_result=[text_result,char(10),['HetEff aperture original = ',num2str(HE_apfield),10,'HetEff treated signal = ',num2str(HE_E),10,'Strehl factor original = ',num2str(strehl_aperture),10,'Strehl factor treated = ',num2str(strehl_corrected)]];
        num_result(it,4)=sum(sum(focus_int(abs(focus_phase_corr_E-full_correct_phase)<0.01)))*focus_sup;
    else
        [~,branchpoints_csm,~,r0_method2]=stadistics_ifield(field, dxa,iangle,'', 0,0 ,0);
        nr_branchpoints_csm_ap = sum(sum(branchpoints_csm.*mask ));
        num_result(it,13)=Scintilation;
        num_result(it,14)=nr_branchpoints_csm_ap;
        num_result(it,15)=r0_method2;
    end
    num_result(it,1)=sum(sum(focus_int))* focus_sup;
    num_result(it,2)=sum(sum(focus_int(best_possible_phase_st~=0)))*focus_sup;
    num_result(it,3)=num_result(it,2)*(fake_metric~=3)+num_result(it,1)*(fake_metric==3);
    num_result(it,5)=sum(sum( focus_int(abs(best_possible_phase_st-full_correct_phase)<0.01)))*focus_sup;
    relevant_data_f{it}=relevant_data;
    row{it}=['F',num2str(base_field) ' NN'];
    if fast~=1
    if (z_coefs>0)&&(~isfield(zern,'aout'))
        [zern.aout{1},phiRec2,zern.rmso(1)]=MatchZernike(GoldsteinUnwrap2D(E_corrected_field),z_coefs);
    end
    end
%     if time
%         save(['./Images/Inputs128/' num2str(iangle) 'degrees/Aperture' num2str(apdiameter_meter) 'm/Inputfield_' num2str(base_field) '.mat'],'field_apmask');
%         save(['./Images/Outputs128/' num2str(iangle) 'degrees/Aperture' num2str(apdiameter_meter) 'm/Outputfield_' num2str(base_field)  '.mat'],'E_corrected_field');
%     end
    if fast~=1
        field_apmask=E_corrected_field;
        focusfield=E_corrected_focus;
    end
    if iterations>1
        row{it}=[row{it} ' It:' num2str(it)];
        focusfield_full = E_corrected_focus_full;
    end
%     save('.\data_out.mat','field_apmask', 'focusfield');
end

%------------------------------ save structure -----------------------------
   
cur_time = fix(clock); 


end

