function [ data ] = APHEFIS_plot( x_axis, y_axis, file,yaxis_operator, condition_name,condition_operator )

%
% APHEFIS_plot('r0_diameter',{'InStrehl','strehl_impr'},0,0,{'r0_diameter','r0_diameter'},{'>0','<6'})
% Function that extracts the information of an stored APHEFIS fields file,
% filters it by then specified conditions, and plot the results in a graphic
% 

if isstruct(file)
    if isfield(file,'APHEFIS_DATA')
        data=file.APHEFIS_DATA;
    else
        data=file;
    end
    FileName='';
elseif ischar(file)&&exist(file,'file')
    FileName=file;
    data=load(file,'-mat');
    data=data.APHEFIS_DATA;
else
    [FileName,PathName,FilterIndex] = uigetfile('.aph','File to Load the Data From:','APHEFIS_DATA.aph');
    [PathName,FileName]
    data=load([PathName,FileName],'-mat');
    data=data.APHEFIS_DATA;
end

if ~iscell(y_axis)
    y_axis={y_axis};
end

if (nargin==3)
    yaxis_operator=0;
end

if (yaxis_operator>2)
    yaxis_operator=2;
end

if (nargin>4)
    if ~iscell(condition_name)
        condition_name={condition_name};
    end
    if ~iscell(condition_operator)
        condition_operator={condition_operator};
    end
    for icnd=1:1:size(condition_name,2)
        if eval(['isfield(data.data_t,',39,'i',condition_name{icnd},39,')'])
            cond=eval(['data.data_t.data(:,data.data_t.i' condition_name{icnd} ')' condition_operator{icnd} ';']);
            cond(isnan(cond))=0;
            data.data_t.rows={data.data_t.rows{cond'}};
            data.data_t.data=data.data_t.data(cond,:);
            data.static_stadistics.data=data.static_stadistics.data(cond,:);
            data.cr_input.data=data.cr_input.data(cond,:);
        elseif eval(['isfield(data.static_stadistics,',39,'i',condition_name{icnd},39,')'])
            cond=eval(['data.static_stadistics.data(:,data.static_stadistics.i' condition_name{icnd} ')' condition_operator{icnd} ';']);
            cond(isnan(cond))=0;
            data.data_t.rows={data.data_t.rows{cond'}};
            data.data_t.data=data.data_t.data(cond,:);
            data.static_stadistics.data=data.static_stadistics.data(cond,:);
            data.cr_input.data=data.cr_input.data(cond,:);
        elseif eval(['isfield(data.cr_input,',39,'i',condition_name{icnd},39,')'])
            cond=eval(['data.cr_input.data(:,data.cr_input.i' condition_name{icnd} ')' condition_operator{icnd} ';']);
            cond(isnan(cond))=0;
            data.data_t.rows={data.data_t.rows{cond'}};
            data.data_t.data=data.data_t.data(cond,:);
            data.static_stadistics.data=data.static_stadistics.data(cond,:);
            data.cr_input.data=data.cr_input.data(cond,:);
        else
            disp(['Wrong condition name:' condition_name{icnd}]);
            return
        end
    end
end

% APHEFIS_DATA=data;
% save('./fields/Filtered/Filtered_data.aph','APHEFIS_DATA');

[ mean_v,median_v,std_deviation,text_table,text_d ] = APHEFIS_characteristics( data,1);


for iaxis=1:1:min(size(y_axis,2),2)
    if eval(['isfield(data.data_t,',39,'i',y_axis{iaxis},39,')'])
            y{iaxis}=eval(['data.data_t.data(:,data.data_t.i' y_axis{iaxis} ');']);
            y{iaxis}(isnan(y{iaxis}))=0;
    elseif eval(['isfield(data.static_stadistics,',39,'i',y_axis{iaxis},39,')'])
            y{iaxis}=eval(['data.static_stadistics.data(:,data.static_stadistics.i' y_axis{iaxis} ');']);
            y{iaxis}(isnan(y{iaxis}))=0;
    elseif eval(['isfield(data.cr_input,',39,'i',y_axis{iaxis},39,')'])
            y{iaxis}=eval(['data.cr_input.data(:,data.cr_input.i' y_axis{iaxis} ');']);
            y{iaxis}(isnan(y{iaxis}))=0;
    else
            disp(['Wrong y_axis:' y_axis{iaxis}]);
            return
    end   
end

if yaxis_operator==0
    lin='none';
end

if eval(['isfield(data.data_t,',39,'i',x_axis,39,')'])
    x=eval(['data.data_t.data(:,data.data_t.i' x_axis ');']);
    x(isnan(x))=0;
elseif eval(['isfield(data.static_stadistics,',39,'i',x_axis,39,')'])
    x=eval(['data.static_stadistics.data(:,data.static_stadistics.i' x_axis ');']);
    x(isnan(x))=0;
elseif eval(['isfield(data.cr_input,',39,'i',x_axis,39,')'])
    x=eval(['data.cr_input.data(:,data.cr_input.i' x_axis ');']);
    x(isnan(x))=0;
elseif strcmp(x_axis,'CDF')
%     keyboard
    for i=1:iaxis
        x_{i}=sort(y{i});
        y{i}=[1:size(x_{i},1)]/size(x_{i},1);
    end
%     keyboard
    x=x_{1};
    lin='-';
elseif strcmp(x_axis,'Histogram')
    for i=1:iaxis
        x_{i}=sort(y{i});
        [y{i},x_{i}]=hist(x_{i});
    end
    x=x_{1};
else
    disp(['Wrong x_axis:' x_axis]);
    return
end



mean_y={};

if yaxis_operator>0
    lin='-';
    xmin=min(x(:));
    index=1;
    jump=(max(x(:))-xmin(:))/50;
    while size(x)>0
        for iaxis=1:1:min(size(y_axis,2),2)
            mean_y{iaxis}(index)=sum(y{iaxis}(((xmin+jump>x).*(x>=xmin))>0))/sum((xmin+jump>x).*(x>=xmin));
            if yaxis_operator>1
                var_y{iaxis}(index)=var(y{iaxis}(((xmin+jump>x).*(x>=xmin))>0));
            end
        end
        x2(index)=xmin;
        x(((xmin+jump>x).*(x>=xmin))>0)=[];
        xmin=min(x);
        index=index+1;
    end
    x=x2;
    if yaxis_operator==1
        y=mean_y;
    elseif yaxis_operator==2
        y=var_y;
    end
else
    
    yaxis_operator=1;
end



while(yaxis_operator>=1)   
    figure;
    hold all
    text('VerticalAlignment','top','HorizontalAlignment','right','Units','normalized','Position',[1.0,1.0],'String',text_d{1,1},'BackgroundColor',[.7 .9 .7]);
    if iaxis~=1
        if exist('x_','var')
            [ax,line1,line2]=plotyy(x,y{1},x_{2},y{2},'plot');
        else
            [ax,line1,line2]=plotyy(x,y{1},x,y{2},'plot');
        end
        grid on;
%         grid(gca,'minor');
        set(get(ax(1),'Ylabel'),'String',transformtitle(y_axis{1})); 
        set(get(ax(2),'Ylabel'),'String',transformtitle(y_axis{2}));
        xlabel(transformtitle(x_axis)); 
        set(line1,'LineStyle',lin)
        set(line2,'LineStyle',lin)
        set(line1,'Marker','o')
        set(line2,'Marker','+')
        set(line1,'MarkerSize',2)
        set(line2,'MarkerSize',2)
        set(gca,'YTick',min(y{1}):(max(y{1})-min(y{1}))/5:max(y{1}))
        axis([min(x) max(x) min(y{1}) max(y{1})])
        axis 'auto x'
    else
        l=plot(x,y{1},'.b');
        set(l,'LineStyle',lin)
        grid on;
%         grid minor;
        set(gca,'MinorGridLineStyle',':')
        xlabel(transformtitle(x_axis));
        ylabel(transformtitle(y_axis{1}));
        min(y_axis{1}):((max(y{1})-min(y{1}))/5):max(y{1})
        set(gca,'YTick',min(y{1}):((max(y{1})-min(y{1}))/5):max(y{1}))
%         axis 'auto y'
    end
    
    if ~strcmp(lin,'none')&&(yaxis_operator==1)
        title('Mean')
    elseif yaxis_operator==2
        title('Variance')
    else
        title(transformtitle(FileName))
    end
    yaxis_operator=yaxis_operator-1;
    y=mean_y;
    hold off
end

end

