function backtrafo_field = backtransformation(focusfield,dxa,lambda,focus,oversampling)
% backtrafo_field = backtransformation(focusfield,dxa,lambda,focus,oversampling)
% 
% The function "backransformation.m" transforms a focusfeld back to the
% aperture field, including the oversampling functionality and removing it again after transformation, input und output field have equal size
% 
% output field has same size as input field - zeropadding is cut away again 
%
% focusfield        complex focusfield
% g_flag            1: graphics are display
% dxa               [m/pix] in the Aperturefield
% lambda            Wavelength [m]
% focus             Focal length [m]
% oversampling      should be the same as "zeropadding" during "focustransformation.m" to get correct scaling
%
% Marion G�del, 30. Juli 2009
% last change: 15. August 2009
% DiGi 20091110 : field_disp entfernt, ovesampling added - correcting the error of wrong scaling
%

ffinit=size(focusfield,1);

focusfield = zeroPadMK(focusfield,size(focusfield,1)*oversampling,0,'center');  % zeroPadding u, factor zwei ist evtl. erforderlich, ansonsten wegstreichen

backtrafo_field = ifftshift( ifft2 (ifftshift(  focusfield * (lambda*focus)/ dxa^2) ) ) ; 

backtrafo_field_full_size = size(backtrafo_field,1); 
 
 backtrafo_field = backtrafo_field((backtrafo_field_full_size - ffinit)/2+1:(backtrafo_field_full_size+ffinit)/2, ...
     (backtrafo_field_full_size-ffinit)/2+1:(backtrafo_field_full_size+ffinit)/2); 
