function [nr_branchpoints_csm, nr_branchpoints_amp,x_csm,y_csm,x_amp,y_amp]= branchpoints_function(compl_inputfield, g_flag)
% [nr_branchpoints_csm, nr_branchpoints_amp,x_csm,y_csm,x_amp,y_amp]= branchpoints_function(compl_inputfield, g_flag)
%
% from BranchPoint.m
% Investigation of Branch Points
% Simulationen von Joachim lambda=1064nm
% MK 28.4.09
%
% Input parameters
% compl_inputfield      complex inputfield
% g_flag                1, if graphics should be displayed
%
%
% Output Parameters
% nr_branchpoints_csm  Number of branchpoints in field, contour sum method
% nr_branchpoints_amp  Number of branchpoints in field, amplitude == 0
% x_csm / y_csm        Coordinate of Branchpoints (contour sum method)
% x_amp / y_amp        Coordinate of Branchpoints (amplitude == 0)
%
% Marion Gdel, June 25, 2009
% last change: August 15, 2009
%
% needed functions: zeroPadMK, (unwrap2, GaussFilter2)



res = size(compl_inputfield,1); 

amplitude = abs(compl_inputfield);     
phase = angle(compl_inputfield);

% sum around a closed loop
p0 = zeroPadMK(phase,[res+1,res+1],0,'lefttop');
px = zeroPadMK(phase,[res+1,res+1],0,'righttop');
py = zeroPadMK(phase,[res+1,res+1],0,'leftbottom');
pxy = zeroPadMK(phase,[res+1,res+1],0,'rightbottom');


dx1 = p0-px; 
dx1 = dx1-(dx1>pi)*2*pi; 
dx1 = dx1+(dx1<-pi)*2*pi;

dx2 = py-pxy; 
dx2 = dx2-(dx2>pi)*2*pi; 
dx2 = dx2+(dx2<-pi)*2*pi;

dy1 = p0-py; 
dy1 = dy1-(dy1>pi)*2*pi; 
dy1 = dy1+(dy1<-pi)*2*pi;

dy2 = pxy-py; 
dy2 = dy2-(dy2>pi)*2*pi; 
dy2 = dy2+(dy2<-pi)*2*pi;

cur = dx1(1:res,2:res+1) + dy1(2:res+1,1:res) + dx2(2:res+1,2:res+1) + dy2(2:res+1,2:res+1);
cur = cur(1:res-1,1:res-1);

interf = sin(rem(phase*5,2*pi));
intLines=((interf>0)&(interf<0.5));
[x,y]=find(intLines==1);

[x_amp,y_amp]=find(amplitude<0.1);
[x_csm,y_csm]=find(abs(cur)>2);
nr_branchpoints_amp = length(y_amp); 
nr_branchpoints_csm = length(y_csm); 


if g_flag
    
    h=figure(123); 
    hold off
    set(h,'Color',[1,1,1]);
    imagesc(phase);
    colorbar
    hold on
    line(y_csm,x_csm,'LineStyle','none','Marker','o','MarkerSize',2);
    line(y_amp,x_amp,'LineStyle','none','Marker','x','MarkerSize',12,'LineWidth',1,'Color','white')
    title('Branchpoints function: Phase of complex inputfield, O: sum around a closed loop > 2, X: amplitude == 0')
    
end




