function rho_i = calculate_rho_int(intensity_matrix,g_flag,dxa)
% rho_i = calculate_rho_int(intensity_matrix,g_flag,dxa)
% 
% Diese Funktion berechnet den Intensitts-Korrelationsradius rho_int. 
% 
% Eingabeparameter
% intensity_matrix          Intensittsmatrix
% g_flag                    1: Graphiken werden angezeigt
%
% Rckgabeparameter
% rho_i                     Intensitts-Korrelationsradius [m]
%
% Marion Gdel, 3. August 2009
% basierend auf einer Berechnung von rho_int von Dirk Giggenbach
%
%
% evalpix:
% Kantenlnge des zentralen Matrixteils, der zur Berechnung herangezogen wird; indem nicht die ganze Matrix durch xcorr2 luft gehts schneller, macht aber nur Sinn, wenn die zu berechnende Gre klein ist gegenber der GesamtMatrix
% Typisch:  =128
% 
% cevalpix:
% Kantenlnge des inneren Teils der Matrix, ber die die 1D-correlation berechnet wird. cevalpix <= evalpix/2
% 
% r_vec:
% r_vec = 0 : round(cevalpix*sqrt(2));
% radius-vector in pixels fr die 1D-corr, starting at zero



evalpix = size(intensity_matrix,1); 
cevalpix = evalpix / 2; 

int_no_mean = intensity_matrix - mean(intensity_matrix(:)); % xcorr2 does not work with double precision

B_int = acf2_Nicotoolbox( int_no_mean );  
b_int = B_int / B_int(evalpix,evalpix); 

% if g_flag
%     figure(200); 
%     imagesc(b_int); shading interp; title('b\_int');
%     title('Autokorrelationsfunktion')
% end



b_int_1D = zeros(round(cevalpix*sqrt(2))+1,1); 
vals_in_r = zeros(round(cevalpix*sqrt(2))+1,1); 
[xmid,ymid] = find(b_int == max(b_int(:))); 

if isnan(max(b_int(:)))
    rho_i = -1; 
    return 
end

for x = -cevalpix : cevalpix
    for y = -cevalpix : cevalpix
        r = round(sqrt(x^2+y^2)+1); % this r=1 means a radius of zero -> shifted right due to vector-indexing
        vals_in_r(r) = vals_in_r(r) + 1; 
        b_int_1D(r) = b_int_1D(r) + b_int(xmid(1) + x, ymid(1) + y); 
    end
end

b_int_1D = b_int_1D ./ vals_in_r; 

% if g_flag
%     figure(100); 
%     r_vec = 0 : round(cevalpix*sqrt(2)); 
%     plot(r_vec * dxa, b_int_1D, '-rd','LineWidth',1); 
% end

rho_i = sum(b_int_1D > exp(-1)) * dxa;      %[m]




 
%  Original: 
%      disp('processing xcov(int)'); clear B_int;
% 
%         B_int = xcorr2(int-meanint(sn)); b_int = B_int /B_int(evalpix,evalpix); % clear B_int;
% 
%         if correlplots figure(200+sn); subplot(2,3,3); imagesc(b_int); shading interp;  title(['b_int bei ',num2str(L/1000),'km']); end
% 
%  
% 
%         b_int_1D  = zeros(round(cevalpix*sqrt(2))+1,1);
% 
%         vals_in_r = zeros(round(cevalpix*sqrt(2))+1,1);
% 
%         [xmid,ymid] = find(b_int==max(b_int(:)));
% 
%         for x=-cevalpix:cevalpix         for y=-cevalpix:cevalpix
% 
%             r = round(sqrt(x^2+y^2)) + 1;  % this r=1 means a radius aof zero -> shifted right due to vector-indexing
% 
%             vals_in_r(r) = vals_in_r(r)+1;
% 
%             b_int_1D(r)  = b_int_1D(r) + b_int(xmid(1)+x , ymid(1)+y);
% 
%         end;    end
% 
%         b_int_1D = b_int_1D ./ vals_in_r;
% 
%         if fdraw figure(100+sn); subplot(2,3,3); plot(r_vec*dx, b_int_1D,'-rd', 'LineWidth',1); end
% 
%         roh_int(sn) = sum(b_int_1D > exp(-1)) * dx;