function [focusfield_cut, dxf] = focustransformation(field_apmask, zeropad_factor, lambda, focus, dxa, focal_cut)
% focusfield = focustransformation(field_apmask,zeropad_factor,lambda,focus,dxa)
% 
% The function focustransformation transformes an inputfield (which already contains its aperture
% mask) into the focus, and cuts the focal matrix to "focal_cut" 
% 
% field_apmask      complex inputfield with aperture mask
% zeropad_factor    factor for zeropadding of the field
% dxa               [m/pix] in the aperture field
% lambda            Wavelength [m]
% focus             focal length [m]
% focal_cut         focal matrix is cut to inner part with size "focal_cut"
% dxf               resolution in meters per pixel of focal matrix

% Marion Gdel, 30. Juli 2009
% DiGi 20091103: modifications (dxa, focal_cut, dxf)
%
% needed functions: zeropadMK

if zeropad_factor < 1
    error('zeropad_factor should be >= 1! ')
end

field_zeropadded = zeroPadMK(field_apmask,size(field_apmask,1)*zeropad_factor,0,'center');  % zeroPadding

focusfield_full = fftshift( fft2 (fftshift(  field_zeropadded) ) )  *  dxa^2 / (lambda*focus); 
focusfield_full_size = size(focusfield_full,1);    
dxf = lambda * focus / (focusfield_full_size*dxa);  % in [meter] per pixel

% zuschneiden des Feldes:
focusfield_cut = focusfield_full((focusfield_full_size-focal_cut)/2+1:(focusfield_full_size+focal_cut)/2,(focusfield_full_size-focal_cut)/2+1:(focusfield_full_size+focal_cut)/2); 
return