function [sf, rho] = getstructf_Nicotoolbox(m, method, AC_weight_mask, roundingflag)
% [sf, rho] = getstructf(m, method, AC_weight_mask, roundingflag)
%
%
% see also ASF2
%
% copied from Nicotoolbox, 30. Juli 2009
% needed functions: rotataverage, acf2, xcf2, asf2


if isempty(AC_weight_mask)
    AC_weight_mask = acf2_Nicotoolbox( ones(size(m)) );
    AC_weight_mask = round(AC_weight_mask); % round to be sure we have absolute zeros
% AC_weight_mask = uint32(AC_weight_mask); 
end

switch method
    
    case 1
        %%%%%%%%%   METHOD 1 %%%%%%%%%%%%%%%%%%%%%%%%%%
        % Method for homogeneous (stationary) field only, e.g. removing the mean over the
        % limited field makes the field inhomogeneous. acf2(f) makes no sense since it
        % averages over a region.
        
        % create a matrix of phase squares (same size as AC_frame)
        sq_frame1 = abs( xcf2_Nicotoolbox(m.^2, ones(size(m))) ); % dont need to round here
        sq_frame2 = rot90(sq_frame1,2); % 180 rotation
        struct_func_2D =  sq_frame1 + sq_frame2 - 2*abs( acf2_Nicotoolbox(m) );
        
        struct_func_2D = struct_func_2D./double(AC_weight_mask);
        
    case 2
        %%%%%%%    METHOD 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % This method ...
        % ... assumes Gaussian statistics.
        % ... is fast but structure function is wrong for high rho values
        %     (rho > 10). Probably because of the 2pi wrapping.
        % ... is valid for inhomogeneous fields

        m = exp(i*m); % complex field
        struct_func_2D = acf2_Nicotoolbox( m ); % assumption of Gaussian statistics
        % The ensemble average should be real but not the estimation
        struct_func_2D = struct_func_2D./double(AC_weight_mask);
        struct_func_2D = -2*log(struct_func_2D);         
        struct_func_2D = real(struct_func_2D);        
        
    case 3
        %%%%%%%    METHOD 3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % It is valid for inhomogeneous fields if the difference
        % field (i.e., f(x2)-f(x1)) is homogeneous.
        struct_func_2D = asf2_Nicotoolbox(m);
        struct_func_2D = struct_func_2D./double(AC_weight_mask);
        
end

[sf, rho] = rotataverage_Nicotoolbox(struct_func_2D, [], roundingflag);
