function [branchpoints_amp,branchpoints_csm,r0_method1,r0_method2]=stadistics_ifield(field, dxa,elevation,label, texts,graphics ,gaussian)
%UNTITLED Summary of this function goes here
%   
%   Calculates the main input field stadistics, and represents them with graphics 
%   and/or text if asked to. The stadistics include:
%   -Scintilation Index
%   -Maximum,Minimum, and Mean Intensity of the Input Field 
%   -r0 values of the wave and phase structure functions 
%   -number of branchpoints
% 
% 
%   "gaussian" tells us if we can consider the field gaussian(2) or if not (1)
%   "texts" and "graphics" tells us if we must display this outputs
%   "compl_inputfield" contains the complex values of the field
%   "dxa" is the relation meter/pixel
%   "elevation" is the angle of elevation at which we are looking
%   "label" is the label of the field, in case we want to put any
%   
%   

posXF=50; posYstartF=900; widthF=1500; heightF=170; posYdeltaF=225;

if gaussian~=2
    [r0Phase_method1,r0_method1] = r0_getstructf_Fried(field,dxa,label,elevation,0,1); % graphics-flag ist "0" %%% nrfield = 0 - is used for figure title only
end
if gaussian~=1
    [r0Phase_method2,r0_method2] = r0_getstructf_Fried(field,dxa,label,elevation,0,2);
end

[nr_branchpoints_csm, nr_branchpoints_amp,x_csm,y_csm,x_amp,y_amp]= branchpoints_function(field, 0);

branchpoints_amp = zeros(size(field,1)); 
branchpoints_csm = zeros(size(field,1)); 

% Matrix with the coordinates of the Branchpoints (1 = Branchpoint)
    
for nr = 1: length(x_amp) 
    branchpoints_amp(x_amp(nr),y_amp(nr)) = 1; 
end
    
for nr = 1: length(x_csm)
    branchpoints_csm(x_csm(nr),y_csm(nr)) = 1; 
end

branchpoints_input = nr_branchpoints_amp; % Improves with MKs function 

intfield = abs(field).^2;

rho_i = calculate_rho_int(intfield,0,dxa); % graphics-flag ist "0"
scintind = mean(mean(intfield.^2)) / mean(intfield(:))^2  - 1 ;     %Scintillation index
min_int = min(intfield(:));
max_int = max(intfield(:));
mean_int = mean(intfield(:));     
P_inputfield = sum(abs(field(:)).^2) * dxa^2; 






%%% ----------------------- Graphics ------------------------- %%%

if graphics
        pixel_inputfield=256;
        pixel_=pixel_inputfield;
        size_field = size(field,1); 
        if size_field<pixel_ 
            pixel_=size_field; 
        end
        zoom_factor = size_field / pixel_; % statt das Feld zu beschneiden wird mit "zoom()" gearbeitet
        
        fignum=100; figure(fignum); 
        upos = [posXF, posYstartF, widthF, heightF]; set(fignum,'Position',upos); set(gcf,'Color',[1 1 1]); %set(gcf,'Colormap', [summer]);
        subplot(1,5,3); imagesc(abs(field).^2); zoom(zoom_factor);colorbar; axis square; title('Inputfield-Intensity');
        % fehlt hier: Aperture-Circles einzeichnen
        subplot(1,5,2); imagesc(abs(field));zoom(zoom_factor); colorbar; axis square; title('Amplitude'); 
        subplot(1,5,1); imagesc(angle(field)); zoom(zoom_factor); colorbar; axis square; caxis manual; caxis([-pi pi]); title('Phase'); 
        hold on; contour(abs(field),1,'k'); hold off;
        subplot(1,5,4); hist(abs(field(:)).^2, 100 ); axis square; title('PDF(I)');
        subplot(1,5,5); deltaYtext=.11; cla; axis off;
        text(-.05,1,'Statistics of Input-Field:'); 
        text(0,1-1*deltaYtext,['dxa = ',num2str(dxa*1000),'mm']);
        text(0,1-2*deltaYtext,['r_0 = 2.1*rho_0: ' num2str(r0_method2),'m']);
        text(0,1-3*deltaYtext,['Scinti-Index = ',num2str(scintind)]); 
        text(0,1-4*deltaYtext,2,['rho-I = ',num2str(rho_i*1000),'mm']);   
        text(0,1-5*deltaYtext,2,['mean(I) = ',num2str(mean_int),'W/m^2']);  
        text(0,1-6*deltaYtext,2,['max(I) = ',num2str(max_int),'W/m^2']);
        text(0,1-7*deltaYtext,2,['Power total field = ',num2str(P_inputfield),'W']);
        text(0,1-8*deltaYtext,2,['Branchpoints total field: ',num2str(branchpoints_input)]);
end
  
%%% ----------------------- Texts ------------------------- %%%
        
if texts
        disp(' ');
        disp('*****************************************************');
        disp('*****************************************************');
        disp( '*** Statistics INPUTFIELD')
        disp(['Scintillation Index: ' num2str(scintind)]); 
        disp(['Minimum Intensity of Inputfield: ' num2str(min_int)])
        disp(['Maximum Intensity of Inputfield: ' num2str(max_int)])
        disp(['Mean Intensity of Inputfield: ' num2str(mean_int)])
        if gaussian~=2
            disp(['r0: Wave-Structure-Function at 6.88 [m]: ' num2str(r0_method1)]); 
        elseif gaussian~=1
            disp(['r0: Phase-Structure-Function at 6.88 [m]: ' num2str(r0Phase_method1)])
        end
        disp(['r0: Wave-Structure-Function at 2, r0 = 2.1*rho0 [m]: ' num2str(r0_method2)])
        disp(['r0: Phase-Structure-Function at 2, r0 = 2.1*rho0 [m]: ' num2str(r0Phase_method2)])        
        disp(['Number of branchpoints (amplitude = 0) : ' num2str(nr_branchpoints_amp)]);
        disp(['Number of branchpoints (contour sum method): ' num2str(nr_branchpoints_csm)]);
        disp(['Intensity Correlation Radius [m]: ' num2str(rho_i)]);
        disp('***')
        disp('*****************************************************');
        disp('*****************************************************');
end


end

