function z = zernimat(M,j)
% Generates the Zernike function of order j in a matrix with resolution M
% M     resolution of the output matrix
% j     order of the Zernike polynomial, 1=piston, 2/3=tilt
%

if j == 1
    z = apmask(M);
    return;
end

rho = distmat(M) *2./M; % radius of 1
theta = angmat(M);


m = zerninum(j);
n = m(:,2);
m = m(:,1);


Rnm = 0;
for S = 0:(n-m)/2
    fac = (-1)^(S)*factorial(n-S)/factorial(S);
    fac = fac/factorial((n+m)/2-S)/factorial((n-m)/2-S);
    Rnm = Rnm + fac*rho.^(n-2*S);
end

z = sqrt(n+1)*Rnm;
if m~=0
    z = z.*sqrt(2) .*cos(pi/2*mod(j,2) - m*theta);
end
z = z.*apmask(M);


% switch j    
%     case 1
%         return;
%     case 2
%         z = 2*rho .* cos(theta); % x tilt
%     case 3
%         z = 2*rho .* sin(theta); % y tilt
%     case 4
%         z = sqrt(3)*(2*rho.^2-1); % defocus
%     case 5
%         z = sqrt(6)*rho.^2 .* sin(2*theta); % x astigmatism
%     case 6
%         z = sqrt(6)*rho.^2 .* cos(2*theta); % y astigmatism
%     case 7
%         z = sqrt(8)*(3*rho.^3-2*rho) .* sin(theta); % x coma
%     case 8
%         z = sqrt(8)*(3*rho.^3-2*rho) .* cos(theta); % y coma
%     case 9
%         z = sqrt(8)*rho.^3 .* sin(3*theta); % x trifoil
%     case 10
%         z = sqrt(8)*rho.^3 .* cos(3*theta); % y trifoil
%     case 11
%         z = sqrt(5)*(6*rho.^4-6*rho.^2+1); % 3rd-order spherical
%     case 12
%         z = sqrt(10)*(4*rho.^4-3*rho.^2) .* cos(2*theta);
%     case 13
%         z = sqrt(10)*(4*rho.^4-3*rho.^2) .* sin(2*theta);
%     case 14
%         z = sqrt(10)*rho.^4 .* cos(4*theta);
%     case 15
%         z = sqrt(10)*rho.^4 .* sin(4*theta);
% end
% 
% z = z.*apmask(M);
